<?xml version="1.0"?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="text" indent="yes" />

  <!--  tables.xsl
        Rick Landau, Dell, 20070310
        From Projector&Display Mgt WG XML for mgt group properties,
        extract a fragment of SNMP MIB syntax. 
        
        In this case, the structural gingerbread around tables.  
        -->

  <xsl:variable name="SNMPprefix">pdm</xsl:variable>
  <xsl:variable name="SNMPPrefix">Pdm</xsl:variable>
  <xsl:variable name="MIBname">pdmMIBObjects</xsl:variable>

  <!-- r o o t -->
  <xsl:template match="/">
    -- Processed by tables11.xsl at . . . <!-- <xsl:value-of select="xsl:current-dateTime()"/> -->

<xsl:apply-templates/>
  </xsl:template>
  
  <!-- G r o u p -->
<xsl:template match="Group">
<!--    - seen:Group <xsl:value-of select="@GroupName"/>\n-->
<xsl:value-of select="$SNMPprefix"/><xsl:value-of select="@GroupName"/> OBJECT IDENTIFIER { <xsl:value-of select="$MIBname"/> inserthere-index }  
    <xsl:apply-templates select="Table">
      <xsl:with-param name="GroupName"><xsl:value-of select="@GroupName"/></xsl:with-param>
    </xsl:apply-templates>
  </xsl:template>
  
  <!-- T a b l e -->
  <xsl:template name="Table" match="Table">
    <xsl:param name="GroupName"/>
<!--    - seen:Table <xsl:value-of select="@TableName"/> in Group <xsl:value-of select="$GroupName"/>\n -->

<!-- What is the name of this table? -->
    <xsl:variable name="mytablenameis">
        <xsl:value-of select="@TableName"/>
    </xsl:variable>

<!-- What is the Index property for this table? -->
    <xsl:variable name="myindexis">
      <xsl:value-of select="$SNMPprefix"/>
      <xsl:value-of select="$GroupName" />Index<!--
 --></xsl:variable>

<!-- What is the description for this table? -->
<!-- Clean up white space and remove double quotes because the
     SNMP DESCRIPTION clause is quoted. -->
    <xsl:variable name="mydescriptionis2">
      <xsl:value-of select="normalize-space(Definition)"/>
    </xsl:variable>
    <xsl:variable name="mydescriptionis">
      <xsl:value-of select="translate($mydescriptionis2,'&quot;',&quot;&apos;&quot;)"/>
    </xsl:variable>

<xsl:text>
</xsl:text>
<xsl:value-of select="$SNMPprefix"/><xsl:value-of select="$mytablenameis"/>
  <xsl:if test="not(contains(@TableName,'Table'))">Table</xsl:if> OBJECT-TYPE -- Table header
    SYNTAX  SEQUENCE OF  <xsl:value-of select="$SNMPPrefix"/><xsl:value-of select="$mytablenameis"/>Entry -- syntax has cap prefix
    MAX-ACCESS not-accessible
    STATUS  current
    DESCRIPTION 
      <xsl:text>"</xsl:text><xsl:value-of select='$mydescriptionis'/><xsl:text>"</xsl:text>
    ::= { <xsl:value-of select="$SNMPprefix"/><xsl:value-of select="$GroupName"/><xsl:text> </xsl:text><xsl:value-of select="position()"/>  }
    
<xsl:value-of select="$SNMPprefix"/><xsl:value-of select="$mytablenameis"/>Entry  OBJECT-TYPE -- Table row
    SYNTAX <xsl:value-of select="$SNMPPrefix"/><xsl:value-of select="$mytablenameis"/>Entry 
    MAX-ACCESS not-accessible
    STATUS  current
    DESCRIPTION ""
    INDEX    { <xsl:value-of select="$myindexis"/> }
    ::= { <xsl:value-of select="$SNMPprefix"/><xsl:value-of select="$mytablenameis"/>
    <xsl:if test="not(contains(@TableName,'Table'))">Table</xsl:if>  1 }

<xsl:value-of select="$SNMPPrefix"/><xsl:value-of select="@TableName"/>Entry -- Columns in table row
    ::= SEQUENCE { 
      <xsl:value-of select="$myindexis"/>   Integer32,<!--
   --><xsl:apply-templates select="Property">
      <xsl:with-param name="GroupName"><xsl:value-of select="$GroupName"/></xsl:with-param>
      <xsl:with-param name="TableName"><xsl:value-of select="@TableName"/></xsl:with-param>
      <xsl:with-param name="EntryName">
        <xsl:value-of select="@TableName"/>
        <xsl:if test="not(contains(@TableName,'Entry'))">Entry</xsl:if>
      </xsl:with-param>
    </xsl:apply-templates>
    }    
  </xsl:template>

  <!-- P r o p e r t y -->
  <xsl:template name="InTableProperty" match="Property">
    <xsl:param name="GroupName"/>
    <xsl:param name="TableName"/>
    <xsl:param name="EntryName"/>

<!--    - Property at-AbstractName=<xsl:value-of select="@AbstractName"/>
    - parent is type=<xsl:value-of select="name(..)"/>
    - Property in table at-TableName=<xsl:value-of select="../@TableName"/>
    - Property in group at-name=<xsl:value-of select="../../@name"/>\n-->
    <xsl:variable name="mynameis">
      <xsl:choose>
        <xsl:when test="contains(SNMPName,' ')">
          <xsl:value-of select="$SNMPprefix"/>
          <xsl:value-of select="$GroupName" />
          <xsl:value-of select="@AbstractName" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$SNMPprefix"/>
          <xsl:value-of select="SNMPName" /></xsl:otherwise>
      </xsl:choose> 
    </xsl:variable>
    
    <!-- If this is a table column, number it within the table entry.
    Otherwise, number it within the containing group. -->
    <xsl:variable name="mycontextis">
      <xsl:choose>
        <xsl:when test="name(..)='Group'">
          <xsl:value-of select="$SNMPprefix"/><xsl:value-of select="$GroupName" />
        </xsl:when>
        <xsl:when test="name(..)='Table'">
          <xsl:value-of select="$SNMPprefix"/>
          <xsl:value-of select="$GroupName" />
          <xsl:value-of select="$EntryName" />
        </xsl:when>
        <xsl:otherwise>ErrorUnknownContext</xsl:otherwise>
      </xsl:choose> 
    </xsl:variable>
    
    <!-- What is the matching data syntax in SNMP? -->
    <xsl:variable name="mysyntaxis">
      <xsl:choose>
        <!-- E N U M -->
        <xsl:when test="Datatype/@Type='EnumeratedInteger'"><xsl:value-of select="$mynameis" />TC</xsl:when>
        <!-- I N T E G E R , maybe with range -->
        <xsl:when test="Datatype/@Type='Integer'"><xsl:text>Integer32</xsl:text>
        </xsl:when>
        <!-- I N T E G E R C O N S T A N T -->
        <xsl:when test="Datatype/@Type='IntegerConstant'"><xsl:text>Integer32</xsl:text></xsl:when>
        <!-- C O U N T E R -->
        <xsl:when test="Datatype/@Type='Counter'">Counter32</xsl:when>
        <!-- S T R I N G , with length -->
        <xsl:when test="Datatype/@Type='String'"><xsl:text>SnmpAdminString</xsl:text>
        </xsl:when>
        <!-- S T R I N G C O N S T A N T , with length -->
        <xsl:when test="Datatype/@Type='StringConstant'"><xsl:text>PdmLocalizedStringTC</xsl:text></xsl:when>
        <!-- G A U G E , with range -->
        <xsl:when test="Datatype/@Type='Gauge'">Gauge32</xsl:when>
        <!-- B O O L E A N -->
        <xsl:when test="Datatype/@Type='Boolean'">TruthValue</xsl:when>
        <!-- U R L -->
        <xsl:when test="Datatype/@Type='URL'"><xsl:text>OCTET STRING </xsl:text></xsl:when>
        <!-- D A T E T I M E -->
        <xsl:when test="Datatype/@Type='AbsoluteDateTime'"><xsl:text>PdmDateTimeTC</xsl:text></xsl:when>
        <!-- T I M E T I C K S -->
        <xsl:when test="Datatype/@Type='TimeTicks'"><xsl:text>TimeTicks  -- SNMP TimeTicks</xsl:text></xsl:when>
        <!-- what else do we use? -->
        <!-- add other datatype translations in here -->
        <!-- unrecognized (yet) -->
        <xsl:otherwise>ErrorUnknownDatatype</xsl:otherwise>
      </xsl:choose> 
    </xsl:variable>
    <xsl:text>
      </xsl:text><xsl:value-of select="$mynameis"/>
      <xsl:text>    </xsl:text><xsl:value-of select="$mysyntaxis" />
    <xsl:if test="not(position()=last())">,</xsl:if>
  </xsl:template>

</xsl:stylesheet>

<!--
Edit history:
20070310    RBL v03 Original usable version.
20070909    RBL v04 Changed naming of table, entry, and entry syntax
                    per comments from Ira McDonald on proper SNMP
                    naming.
20070910    RBL v05 Fix formatting for improved legibility. 
20070913    RBL v06 Add Index property to the beginning of any table. 
20071009    RBL v08 Correct printed breadcrumbs version number.
20071024    RBL v09 Add URL datatype.
20071028    RBL v10 Add DateTime and TimeTicks datatypes. 
20071030    RBL v11 Correct spelling of DateTime to AbsoluteDateTime. 

-->
